@@ -93,7 +93,7 @@ wait_child (PasswordDialog *pdialog)
 	if (pid > 0) {
 
 		if (WIFEXITED (status) && (WEXITSTATUS(status) == 0)) {
-			gtk_main_quit ();
+			/* I need to quit here */
 
 			return FALSE;
 		} else if ((WIFEXITED (status)) && (WEXITSTATUS (status)) && (WEXITSTATUS(status) < 255)) {
@@ -161,7 +161,8 @@ read_everything (PasswordDialog *pdialog, gchar *needle, va_list ap)
 {
 	GString *str  = g_string_new ("");
 	GSList  *list = NULL;
-	gchar*arg, *ptr, c;
+	gchar*arg, *ptr;
+	int c;
 
 	list = g_slist_prepend (list, needle);
   
@@ -260,7 +261,7 @@ passdlg_set_busy (PasswordDialog *pdialog, gboolean busy)
 
 
 static gint
-update_password (PasswordDialog *pdialog) 
+update_password (PasswordDialog *pdialog, gchar **msg)
 {
 	GtkWidget *wopasswd, *wnpasswd, *wrnpasswd;
 	char *new_password;
@@ -268,6 +269,7 @@ update_password (PasswordDialog *pdialog)
 	char *old_password;
 	gchar *s;
 	GladeXML *dialog;
+	gint retcode;
 	
 	dialog = pdialog->xml;
 	
@@ -275,6 +277,8 @@ update_password (PasswordDialog *pdialog)
 	wnpasswd = WID ("new-password");
 	wrnpasswd = WID ("retyped-password");
 
+	retcode = 0;
+
 	/* */
 	old_password = g_strdup_printf ("%s\n", gtk_entry_get_text (GTK_ENTRY (wopasswd)));
 	new_password = g_strdup_printf ("%s\n", gtk_entry_get_text (GTK_ENTRY (wnpasswd)));
@@ -304,17 +308,33 @@ update_password (PasswordDialog *pdialog)
 		
 	write_to_backend (pdialog, retyped_password);
 	
-	s = read_from_backend (pdialog, "successfully", "Bad:", "recovered",  NULL);
+	s = read_from_backend (pdialog, "successfully", "short", "panlindrome", "simple", "similar", "wrapped", "recovered",  "unchanged", NULL);
 	if (g_strrstr (s, "recovered") != NULL) {
-		return -2;
-	} else if (g_strrstr (s, "Bad") != NULL) {
-		return -3;
+		retcode = -2;
+	} else if (g_strrstr (s, "short") != NULL) {
+		*msg = g_strdup (_("Password is too short"));
+		retcode = -3;
+	} else if (g_strrstr (s, "panlindrome") != NULL) {
+		*msg = g_strdup (_("Password is too simple"));
+		retcode = -3;
+	} else if (g_strrstr (s, "simple") != NULL) {
+		*msg = g_strdup (_("Password is too simple"));
+		retcode = -3;
+	} else if ((g_strrstr (s, "similar") != NULL) || (g_strrstr (s, "wrapped") != NULL)) {
+		*msg = g_strdup (_("Old and new passwords are too similar"));
+		retcode = -3;
+	} else if (g_strrstr (s, "unchanged") != NULL) {
+		kill (pdialog->backend_pid, SIGKILL);
+		*msg = g_strdup (_("Old and new password are the same"));
+		retcode = -3;
 	}
 
-	return 0;
+	g_free (s);
+	
+	return retcode;
 }
 
-static void
+static gint
 spawn_passwd (PasswordDialog *pdialog)
 {
 	char *args[2];
@@ -328,7 +348,7 @@ spawn_passwd (PasswordDialog *pdialog)
 	pdialog->backend_pid = forkpty (&pdialog->write_fd, NULL, NULL, NULL);
 	if (pdialog->backend_pid < 0) {
 		g_warning ("could not fork to backend");
-		gtk_main_quit ();
+		return -1;
 	} else if (pdialog->backend_pid == 0) {
 		dup2 (p[1], 1);
 		dup2 (p[1], 2);
@@ -352,65 +372,17 @@ spawn_passwd (PasswordDialog *pdialog)
 		setvbuf (pdialog->read_stream, NULL, _IONBF, 0);
 		fcntl (pdialog->read_fd, F_SETFL, 0);
 	}
+
+	return 1;
 }
 
-static void
-passdlg_button_clicked_cb (GtkDialog *widget, gint response_id, PasswordDialog *pdialog) 
-{
-	GladeXML *dialog;
-	GtkWidget *wmessage, *wbulb;
-	gchar *msg;
-	gint ret;
-
-	dialog = pdialog->xml;
-
-	wmessage = WID ("message");
-	wbulb = WID ("bulb");
-
-	if (response_id == GTK_RESPONSE_OK) {
-		spawn_passwd (pdialog);
-
-		ret = update_password (pdialog);
-		passdlg_set_busy (pdialog, FALSE);
-
-		/* No longer need the wait_child fallback, remove the timeout */
-		g_source_remove (pdialog->timeout_id);
-
-		if (ret == -1) {
-			msg = g_strdup_printf ("<b>%s</b>", _("Old password is incorrect, please retype it"));
-			gtk_label_set_markup (GTK_LABEL (wmessage), msg);
-			g_free (msg);
-
-			gtk_image_set_from_file (GTK_IMAGE (wbulb),
-						 GNOMECC_DATA_DIR "/pixmaps/gnome-about-me-bulb-off.png");
-
-		}
-
-		/* This is the standard way of returning from the dialog with passwd 
-		 * If we return this way we can safely kill passwd as it has completed
-		 * its task. In case of problems we still have the wait_child fallback
-		 */
-		fclose (pdialog->write_stream);
-		fclose (pdialog->read_stream);
-		
-		close (pdialog->read_fd);
-		close (pdialog->write_fd);
-
-		kill (pdialog->backend_pid, 9);
-
-		if (ret == 0)
-			gtk_main_quit ();
-	} else {
-		gtk_main_quit ();
-	}
-}
 
 static gboolean
 passdlg_check_password_timeout_cb (PasswordDialog *pdialog)
 {
 	const gchar *password;
 	const gchar *retyped_password;
-	char *msg, *msgtmp;
+	char *msg;
 	gboolean good_password;
 
 	GtkWidget *wbulb, *wok, *wmessage;
@@ -442,7 +414,7 @@ passdlg_check_password_timeout_cb (PasswordDialog *pdialog)
 		msg = g_strconcat ("<b>", _("Please type the password again, it is wrong."), "</b>", NULL);
 		good_password = FALSE;
 	} else {
-		msg = g_strconcat ("<b>", _("Click the Change Password button to change the password."), "</b>", NULL);
+		msg = g_strconcat ("<b>", _("Click on Change Password to change the password."), "</b>", NULL);
 		good_password = TRUE;
 	}
 
@@ -473,16 +445,84 @@ passdlg_check_password (GtkEntry *entry, PasswordDialog *pdialog)
 	}
 
 	pdialog->check_password_timeout_id =
-		g_timeout_add (300, (GSourceFunc) passdlg_check_password_timeout_cb, pdialog);
+		g_timeout_add (500, (GSourceFunc) passdlg_check_password_timeout_cb, pdialog);
 	
 }
 
+static gint
+passdlg_process_response (PasswordDialog *pdialog, gint response_id) 
+{
+	GladeXML *dialog;
+	GtkWidget *wmessage, *wbulb;
+	gchar *msg, *msgerr;
+	gint ret;
+
+	dialog = pdialog->xml;
+
+	wmessage = WID ("message");
+	wbulb = WID ("bulb");
+
+	msgerr = NULL;
+
+	if (response_id == GTK_RESPONSE_OK) {
+		ret = spawn_passwd (pdialog);
+		if (ret < 0)
+			return 1;
+
+		ret = update_password (pdialog, &msgerr);
+		passdlg_set_busy (pdialog, FALSE);
+
+		/* No longer need the wait_child fallback, remove the timeout */
+		g_source_remove (pdialog->timeout_id);
+
+		if (ret == -1) {
+			msg = g_strdup_printf ("<b>%s</b>", _("Old password is incorrect, please retype it"));
+			gtk_label_set_markup (GTK_LABEL (wmessage), msg);
+			g_free (msg);
+
+			gtk_image_set_from_file (GTK_IMAGE (wbulb),
+						 GNOMECC_DATA_DIR "/pixmaps/gnome-about-me-bulb-off.png");
+
+			return -1;
+		} else if (ret == -3) {
+			msg = g_strdup_printf ("<b>%s</b>", msgerr);
+			gtk_label_set_markup (GTK_LABEL (wmessage), msg);
+			g_free (msg);
+			
+			gtk_image_set_from_file (GTK_IMAGE (wbulb),
+						 GNOMECC_DATA_DIR "/pixmaps/gnome-about-me-bulb-off.png");
+						 
+			g_free (msgerr);
+			
+			return -1;
+		}
+
+		/* This is the standard way of returning from the dialog with passwd 
+		 * If we return this way we can safely kill passwd as it has completed
+		 * its task. In case of problems we still have the wait_child fallback
+		 */
+		fclose (pdialog->write_stream);
+		fclose (pdialog->read_stream);
+		
+		close (pdialog->read_fd);
+		close (pdialog->write_fd);
+
+		kill (pdialog->backend_pid, 9);
+
+		if (ret == 0)
+			return 1;
+	} else {
+		return 1;
+	}
+}
+
 void
 gnome_about_me_password (GtkWindow *parent)
 {
 	PasswordDialog *pdialog;
 	GtkWidget *wpassdlg;
 	GladeXML *dialog;
+	gint result;
 	
 	pdialog = g_new0 (PasswordDialog, 1);
 
@@ -492,8 +532,6 @@ gnome_about_me_password (GtkWindow *parent)
 
 	wpassdlg = WID ("change-password");
 	capplet_set_icon (wpassdlg, "user-info");
-	g_signal_connect (G_OBJECT (wpassdlg), "response",
-			  G_CALLBACK (passdlg_button_clicked_cb), pdialog);
 
 	pdialog->good_password = FALSE;
 
@@ -508,7 +546,12 @@ gnome_about_me_password (GtkWindow *parent)
 	gtk_window_set_resizable (GTK_WINDOW (wpassdlg), FALSE);
 	gtk_window_set_transient_for (GTK_WINDOW (wpassdlg), GTK_WINDOW (parent));
 	gtk_widget_show_all (wpassdlg);
-	gtk_main ();
+
+
+	do {
+		result = gtk_dialog_run (GTK_DIALOG (wpassdlg));
+		result = passdlg_process_response (pdialog, result);
+	} while (result <= 0);
 
 	gtk_widget_destroy (wpassdlg);
 	g_free (pdialog);
