@@ -44,7 +44,7 @@ struct DisplayInfo {
 static void generate_rate_menu (struct ScreenInfo *screen_info);
 static void generate_resolution_menu(struct ScreenInfo* screen_info);
 
-struct DisplayInfo *
+static struct DisplayInfo *
 read_display_info (GdkDisplay *display)
 {
   struct DisplayInfo *info;
@@ -77,7 +77,7 @@ read_display_info (GdkDisplay *display)
   return info;
 }
 
-void
+static void
 update_display_info (struct DisplayInfo *info, GdkDisplay *display)
 {
   struct ScreenInfo *screen_info;
@@ -191,10 +191,17 @@ apply_config (struct DisplayInfo *info)
 
   update_display_info (info, display);
   
-  /* xscreensaver should handle this itself, but does not currently so we hack
-   * it.  Ignore failures in case xscreensaver is not installed */
-  if (changed)
-   g_spawn_command_line_async ("xscreensaver-command -restart", NULL);
+  if (changed) {
+    gchar *cmd;
+
+    if ((cmd = g_find_program_in_path ("gnome-screensaver-command")))
+      g_free (cmd);
+    else {
+      /* xscreensaver should handle this itself, but does not currently so we hack
+       * it.  Ignore failures in case xscreensaver is not installed */
+      g_spawn_command_line_async ("xscreensaver-command -restart", NULL);
+    }
+  }
 
   return changed;
 }
@@ -206,6 +213,7 @@ revert_config (struct DisplayInfo *info)
   GdkDisplay *display;
   Display *xdisplay;
   GdkScreen *screen;
+  char *cmd;
 
   display = gdk_display_get_default ();
   xdisplay = gdk_x11_display_get_xdisplay (display);
@@ -240,10 +248,14 @@ revert_config (struct DisplayInfo *info)
       generate_rate_menu (screen_info);
     }
 
-  /* xscreensaver should handle this itself, but does not currently so we hack
-   * it.  Ignore failures in case xscreensaver is not installed */
-  g_spawn_command_line_async ("xscreensaver-command -restart", NULL);
-  
+  if ((cmd = g_find_program_in_path ("gnome-screensaver-command")))
+    g_free (cmd);
+  else {
+    /* xscreensaver should handle this itself, but does not currently so we hack
+     * it.  Ignore failures in case xscreensaver is not installed */
+    g_spawn_command_line_async ("xscreensaver-command -restart", NULL);
+  }
+
   return 0;
 }
 
@@ -574,13 +586,13 @@ struct TimeoutData {
   gboolean timed_out;
 };
 
-char *
+static char *
 timeout_string (int time)
 {
   return g_strdup_printf (ngettext ("Testing the new settings. If you don't respond in %d second the previous settings will be restored.", "Testing the new settings. If you don't respond in %d seconds the previous settings will be restored.", time), time);
 }
 
-gboolean
+static gboolean
 save_timeout_callback (gpointer _data)
 {
   struct TimeoutData *data = _data;
