@@ -6,11 +6,14 @@
 #include <bonobo/bonobo-object.h>
 #include <bonobo/bonobo-generic-factory.h>
 
+#include <gconf/gconf.h>
+
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-client.h>
 
 static BonoboObject *services_server = NULL;
+GConfClient *conf_client = NULL;
 
 int main (int argc, char *argv [])
 {
@@ -55,5 +58,9 @@ int main (int argc, char *argv [])
 
   gtk_main();
 
+  /* cleanup */
+  if (conf_client)
+	  g_object_unref (conf_client);
+
   return -1;
-}                                                                             
+}
